@@ -18,6 +18,7 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
+from weakref import ref
 import gc
 
 from storm.exceptions import ClassInfoError
@@ -26,7 +27,7 @@ from storm.variables import Variable
 from storm.expr import Undef, Select, compile
 from storm.info import *
 
-from tests.helper import TestHelper
+from tests.helper import TestHelper, run_this
 
 
 class Wrapper(object):
@@ -205,10 +206,25 @@ class ObjectInfoTest(TestHelper):
         for column in self.cls_info.columns:
             variable = self.obj_info.variables.get(column)
             self.assertTrue(isinstance(variable, Variable))
+            self.assertTrue(variable.column is column)
 
         self.assertEquals(len(self.obj_info.variables),
                           len(self.cls_info.columns))
 
+    def test_variable_has_validator_object_factory(self):
+        args = []
+        def validator(obj, attr, value):
+            args.append((obj, attr, value))
+        class Class(object):
+            __storm_table__ = "table"
+            prop = Property(primary=True,
+                            variable_kwargs={"validator": validator})
+
+        obj = Class()
+        get_obj_info(obj).variables[Class.prop].set(123)
+
+        self.assertEquals(args, [(obj, "prop", 123)])
+
     def test_primary_vars(self):
         self.assertTrue(isinstance(self.obj_info.primary_vars, tuple))
         
@@ -449,6 +465,20 @@ class ObjectInfoTest(TestHelper):
     def test_get_obj(self):
         self.assertTrue(self.obj_info.get_obj() is self.obj)
 
+    def test_get_obj_reference(self):
+        """
+        We used to assign the get_obj() manually. This breaks stored
+        references to the method (IOW, what we do in the test below).
+
+        It was a bit faster, but in exchange for the danger of introducing
+        subtle bugs which are super hard to debug.
+        """
+        get_obj = self.obj_info.get_obj
+        self.assertTrue(get_obj() is self.obj)
+        another_obj = self.Class()
+        self.obj_info.set_obj(another_obj)
+        self.assertTrue(get_obj() is another_obj)
+
     def test_set_obj(self):
         obj = self.Class()
         self.obj_info.set_obj(obj)
@@ -497,7 +527,6 @@ class ClassAliasTest(TestHelper):
             __storm_table__ = "table"
             prop1 = Property("column1", primary=True)
         self.Class = Class
-        self.obj = Class()
         self.ClassAlias = ClassAlias(self.Class, "alias")
         
     def test_cls_info_cls(self):
@@ -532,6 +561,66 @@ class ClassAliasTest(TestHelper):
                           'SELECT "select".column1 FROM "table" AS "select" '
                           'WHERE "select".column1 = ?')
 
+    def test_crazy_metaclass(self):
+        """We don't want metaclasses playing around when we build an alias."""
+        TestHelper.setUp(self)
+        class MetaClass(type):
+            def __new__(meta_cls, name, bases, dict):
+                cls = type.__new__(meta_cls, name, bases, dict)
+                cls.__storm_table__ = "HAH! GOTCH YA!"
+                return cls
+        class Class(object):
+            __metaclass__ = MetaClass
+            __storm_table__ = "table"
+            prop1 = Property("column1", primary=True)
+        Alias = ClassAlias(Class, "USE_THIS")
+        self.assertEquals(Alias.__storm_table__, "USE_THIS")
+
+    def test_cached_aliases(self):
+        """
+        Class aliases are cached such that multiple invocations of
+        C{ClassAlias} return the same object.
+        """
+        alias1 = ClassAlias(self.Class, "something_unlikely")
+        alias2 = ClassAlias(self.Class, "something_unlikely")
+        self.assertIdentical(alias1, alias2)
+        alias3 = ClassAlias(self.Class, "something_unlikely2")
+        self.assertNotIdentical(alias1, alias3)
+        alias4 = ClassAlias(self.Class, "something_unlikely2")
+        self.assertIdentical(alias3, alias4)
+
+    def test_unnamed_aliases_not_cached(self):
+        alias1 = ClassAlias(self.Class)
+        alias2 = ClassAlias(self.Class)
+        self.assertNotIdentical(alias1, alias2)
+
+    def test_alias_cache_is_per_class(self):
+        """
+        The cache of class aliases is not as bad as it once was.
+        """
+        class LocalClass(self.Class):
+            pass
+        alias = ClassAlias(self.Class, "something_unlikely")
+        alias2 = ClassAlias(LocalClass, "something_unlikely")
+        self.assertNotIdentical(alias, alias2)
+
+    def test_aliases_only_last_as_long_as_class(self):
+        """
+        The cached ClassAliases only last for as long as the class is alive.
+        """
+        class LocalClass(self.Class):
+            pass
+        alias = ClassAlias(LocalClass, "something_unlikely3")
+        alias_ref = ref(alias)
+        class_ref = ref(LocalClass)
+        del alias
+        del LocalClass
+
+        gc.collect(); gc.collect(); gc.collect()
+
+        self.assertIdentical(class_ref(), None)
+        self.assertIdentical(alias_ref(), None)
+
 
 class TypeCompilerTest(TestHelper):
 
@@ -548,3 +637,4 @@ class TypeCompilerTest(TestHelper):
         alias = ClassAlias(Class2, "alias")
         statement = compile(Select(alias.id))
         self.assertEquals(statement, "SELECT alias.id FROM class1 AS alias")
+
